<?php
/**
 * Listeners
 *
 * @package GamiPress\LearnDash\Listeners
 * @since 1.0.0
 */

// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

/**
 * Complete quiz
 *
 * @since 1.0.0
 *
 * @param array $quiz_data array(
 *      'course' => WP_Post,
 *      'quiz' => WP_Post,
 *      'pass' => integer,
 *      'percentage' => integer,
 * )
 * @param WP_User $current_user
 */
function gamipress_ld_complete_quiz( $quiz_data, $current_user ) {

    $course_id = $quiz_data['course'] instanceof WP_Post ? $quiz_data['course']->ID : 0;

    // Complete any quiz
    do_action( 'gamipress_ld_complete_quiz', $quiz_data['quiz']->ID, $current_user->ID, $course_id, $quiz_data );

    // Complete specific quiz
    do_action( 'gamipress_ld_complete_specific_quiz', $quiz_data['quiz']->ID, $current_user->ID, $course_id, $quiz_data );

    // Complete any quiz with a minimum percent grade
    do_action( 'gamipress_ld_complete_quiz_grade', $quiz_data['quiz']->ID, $current_user->ID, $course_id, absint( $quiz_data['percentage'] ), $quiz_data );

    // Complete specific quiz with a minimum percent grade
    do_action( 'gamipress_ld_complete_specific_quiz_grade', $quiz_data['quiz']->ID, $current_user->ID, $course_id, absint( $quiz_data['percentage'] ), $quiz_data );

    // Complete any quiz with a maximum percent grade
    do_action( 'gamipress_ld_complete_quiz_max_grade', $quiz_data['quiz']->ID, $current_user->ID, $course_id, absint( $quiz_data['percentage'] ), $quiz_data );

    // Complete specific quiz with a maximum percent grade
    do_action( 'gamipress_ld_complete_specific_quiz_max_grade', $quiz_data['quiz']->ID, $current_user->ID, $course_id, absint( $quiz_data['percentage'] ), $quiz_data );

    // If user has successfully passed the quiz
    if( $quiz_data['pass'] ) {

        // Pass any quiz
        do_action( 'gamipress_ld_pass_quiz', $quiz_data['quiz']->ID, $current_user->ID, $course_id, $quiz_data );

        // Pass specific quiz
        do_action( 'gamipress_ld_pass_specific_quiz', $quiz_data['quiz']->ID, $current_user->ID, $course_id, $quiz_data );

    } else {
        // User has failed the quiz

        // Fail any quiz
        do_action( 'gamipress_ld_fail_quiz', $quiz_data['quiz']->ID, $current_user->ID, $course_id, $quiz_data );

        // Fail specific quiz
        do_action( 'gamipress_ld_fail_specific_quiz', $quiz_data['quiz']->ID, $current_user->ID, $course_id, $quiz_data );
    }

}
add_action( 'learndash_quiz_completed', 'gamipress_ld_complete_quiz', 10, 2 );

/**
 * Complete topic
 *
 * @since 1.0.0
 *
 * @param array $args array(
 *      'user' => WP_User,
 *      'course' => WP_Post,
 *      'lesson' => WP_Post,
 *      'topic' => WP_Post,
 *      'progress' => array,
 * )
 */
function gamipress_ld_complete_topic( $args ) {

    // Complete any topic
    do_action( 'gamipress_ld_complete_topic', $args['topic']->ID, $args['user']->ID, $args['lesson']->ID, $args['course']->ID, $args );

    // Complete specific topic
    do_action( 'gamipress_ld_complete_specific_topic', $args['topic']->ID, $args['user']->ID, $args['lesson']->ID, $args['course']->ID, $args );

}
add_action( 'learndash_topic_completed', 'gamipress_ld_complete_topic' );

/**
 * Complete lesson
 *
 * @since 1.0.0
 *
 * @param array $args array(
 *      'user' => WP_User,
 *      'course' => WP_Post,
 *      'lesson' => WP_Post,
 *      'progress' => array,
 * )
 */
function gamipress_ld_complete_lesson( $args ) {

    // Complete any lesson
    do_action( 'gamipress_ld_complete_lesson', $args['lesson']->ID, $args['user']->ID, $args['course']->ID, $args );

    // Complete specific lesson
    do_action( 'gamipress_ld_complete_specific_lesson', $args['lesson']->ID, $args['user']->ID, $args['course']->ID, $args );

}
add_action( 'learndash_lesson_completed', 'gamipress_ld_complete_lesson' );

/**
 * Complete course
 *
 * @since 1.0.0
 *
 * @param array $args array(
 *      'user' => WP_User,
 *      'course' => WP_Post,
 *      'progress' => array,
 * )
 */
function gamipress_ld_complete_course( $args ) {

    // Complete any course
    do_action( 'gamipress_ld_complete_course', $args['course']->ID, $args['user']->ID, $args );

    // Complete specific course
    do_action( 'gamipress_ld_complete_specific_course', $args['course']->ID, $args['user']->ID, $args );

}
add_action( 'learndash_course_completed', 'gamipress_ld_complete_course' );
