<?php
/**
 * Triggers
 *
 * @package GamiPress\LearnDash\Triggers
 * @since 1.0.0
 */

// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

/**
 * Register LearnDash specific triggers
 *
 * @param array $triggers
 * @return mixed
 */
function gamipress_ld_activity_triggers( $triggers ) {

    $triggers[__( 'LearnDash', 'gamipress-learndash-integration' )] = array(
        'gamipress_ld_complete_quiz' => __( 'Complete a quiz', 'gamipress-learndash-integration' ),
        'gamipress_ld_complete_specific_quiz' => __( 'Complete a specific quiz', 'gamipress-learndash-integration' ),

        'gamipress_ld_complete_quiz_grade' => __( 'Complete a quiz with a minimum percent grade', 'gamipress-learndash-integration' ),
        'gamipress_ld_complete_specific_quiz_grade' => __( 'Complete a specific quiz with a minimum percent grade', 'gamipress-learndash-integration' ),

        'gamipress_ld_complete_quiz_max_grade' => __( 'Complete a quiz with a maximum percent grade', 'gamipress-learndash-integration' ),
        'gamipress_ld_complete_specific_quiz_max_grade' => __( 'Complete a specific quiz with a maximum percent grade', 'gamipress-learndash-integration' ),

        'gamipress_ld_pass_quiz' => __( 'Successfully pass a quiz', 'gamipress-learndash-integration' ),
        'gamipress_ld_pass_specific_quiz' => __( 'Successfully pass a specific quiz', 'gamipress-learndash-integration' ),

        'gamipress_ld_fail_quiz' => __( 'Fail a quiz', 'gamipress-learndash-integration' ),
        'gamipress_ld_fail_specific_quiz' => __( 'Fail a specific quiz', 'gamipress-learndash-integration' ),

        'gamipress_ld_complete_topic' => __( 'Complete a topic', 'gamipress-learndash-integration' ),
        'gamipress_ld_complete_specific_topic' => __( 'Complete a specific topic', 'gamipress-learndash-integration' ),

        'gamipress_ld_complete_lesson' => __( 'Complete a lesson', 'gamipress-learndash-integration' ),
        'gamipress_ld_complete_specific_lesson' => __( 'Complete a specific lesson', 'gamipress-learndash-integration' ),

        'gamipress_ld_complete_course' => __( 'Complete a course', 'gamipress-learndash-integration' ),
        'gamipress_ld_complete_specific_course' => __( 'Complete a specific course', 'gamipress-learndash-integration' ),
    );

    return $triggers;

}
add_filter( 'gamipress_activity_triggers', 'gamipress_ld_activity_triggers' );

/**
 * Register LearnDash specific activity triggers
 *
 * @since  1.0.0
 *
 * @param  array $specific_activity_triggers
 * @return array
 */
function gamipress_ld_specific_activity_triggers( $specific_activity_triggers ) {

    $specific_activity_triggers['gamipress_ld_complete_specific_quiz'] = array( 'sfwd-quiz' );
    $specific_activity_triggers['gamipress_ld_complete_specific_quiz_grade'] = array( 'sfwd-quiz' );
    $specific_activity_triggers['gamipress_ld_complete_specific_quiz_max_grade'] = array( 'sfwd-quiz' );
    $specific_activity_triggers['gamipress_ld_pass_specific_quiz'] = array( 'sfwd-quiz' );
    $specific_activity_triggers['gamipress_ld_fail_specific_quiz'] = array( 'sfwd-quiz' );
    $specific_activity_triggers['gamipress_ld_complete_specific_topic'] = array( 'sfwd-topic' );
    $specific_activity_triggers['gamipress_ld_complete_specific_lesson'] = array( 'sfwd-lessons' );
    $specific_activity_triggers['gamipress_ld_complete_specific_course'] = array( 'sfwd-courses' );

    return $specific_activity_triggers;
}
add_filter( 'gamipress_specific_activity_triggers', 'gamipress_ld_specific_activity_triggers' );

/**
 * Build custom activity trigger label
 *
 * @param string    $title
 * @param integer   $requirement_id
 * @param array     $requirement
 *
 * @return string
 */
function gamipress_ld_activity_trigger_label( $title, $requirement_id, $requirement ) {

    $minimum_score = ( isset( $requirement['ld_score'] ) ) ? absint( $requirement['ld_score'] ) : 0;

    switch( $requirement['trigger_type'] ) {
        case 'gamipress_ld_complete_quiz_grade':
            return sprintf( __( 'Completed a quiz with a score of %d or higher', 'gamipress-learndash-integration' ), $minimum_score );
            break;
        case 'gamipress_ld_complete_specific_quiz_grade':
            $achievement_post_id = absint( $requirement['achievement_post'] );
            return sprintf( __( 'Complete the quiz %s with a score of %d or higher', 'gamipress-learndash-integration' ), get_the_title( $achievement_post_id ), $minimum_score );
            break;
        case 'gamipress_ld_complete_quiz_max_grade':
            return sprintf( __( 'Completed a quiz with a maximum score of %d', 'gamipress-learndash-integration' ), $minimum_score );
            break;
        case 'gamipress_ld_complete_specific_quiz_max_grade':
            $achievement_post_id = absint( $requirement['achievement_post'] );
            return sprintf( __( 'Complete the quiz %s with a maximum score of %d', 'gamipress-learndash-integration' ), get_the_title( $achievement_post_id ), $minimum_score );
            break;
    }

    return $title;
}
add_filter( 'gamipress_activity_trigger_label', 'gamipress_ld_activity_trigger_label', 10, 3 );

/**
 * Register LearnDash specific activity triggers labels
 *
 * @since  1.0.0
 *
 * @param  array $specific_activity_trigger_labels
 * @return array
 */
function gamipress_ld_specific_activity_trigger_label( $specific_activity_trigger_labels ) {

    $specific_activity_trigger_labels['gamipress_ld_complete_specific_quiz'] = __( 'Complete the quiz %s', 'gamipress-learndash-integration' );
    $specific_activity_trigger_labels['gamipress_ld_pass_specific_quiz'] = __( 'Pass the quiz %s', 'gamipress-learndash-integration' );
    $specific_activity_trigger_labels['gamipress_ld_fail_specific_quiz'] = __( 'Fail the quiz %s', 'gamipress-learndash-integration' );
    $specific_activity_trigger_labels['gamipress_ld_complete_specific_topic'] = __( 'Complete the topic %s', 'gamipress-learndash-integration' );
    $specific_activity_trigger_labels['gamipress_ld_complete_specific_lesson'] = __( 'Complete the lesson %s', 'gamipress-learndash-integration' );
    $specific_activity_trigger_labels['gamipress_ld_complete_specific_course'] = __( 'Complete the course %s', 'gamipress-learndash-integration' );

    return $specific_activity_trigger_labels;
}
add_filter( 'gamipress_specific_activity_trigger_label', 'gamipress_ld_specific_activity_trigger_label' );

/**
 * Get user for a given trigger action.
 *
 * @since  1.0.0
 *
 * @param  integer $user_id user ID to override.
 * @param  string  $trigger Trigger name.
 * @param  array   $args    Passed trigger args.
 * @return integer          User ID.
 */
function gamipress_ld_trigger_get_user_id( $user_id, $trigger, $args ) {

    switch ( $trigger ) {
        case 'gamipress_ld_complete_quiz':
        case 'gamipress_ld_complete_specific_quiz':
        case 'gamipress_ld_complete_quiz_grade':
        case 'gamipress_ld_complete_specific_quiz_grade':
        case 'gamipress_ld_complete_quiz_max_grade':
        case 'gamipress_ld_complete_specific_quiz_max_grade':
        case 'gamipress_ld_pass_quiz':
        case 'gamipress_ld_pass_specific_quiz':
        case 'gamipress_ld_fail_quiz':
        case 'gamipress_ld_fail_specific_quiz':
        case 'gamipress_ld_complete_topic':
        case 'gamipress_ld_complete_specific_topic':
        case 'gamipress_ld_complete_lesson':
        case 'gamipress_ld_complete_specific_lesson':
        case 'gamipress_ld_complete_course':
        case 'gamipress_ld_complete_specific_course':
            $user_id = $args[1];
            break;
    }

    return $user_id;

}
add_filter( 'gamipress_trigger_get_user_id', 'gamipress_ld_trigger_get_user_id', 10, 3 );

/**
 * Get the id for a given specific trigger action.
 *
 * @since  1.0.0
 *
 * @param  integer  $specific_id Specific ID.
 * @param  string  $trigger Trigger name.
 * @param  array   $args    Passed trigger args.
 *
 * @return integer          Specific ID.
 */
function gamipress_ld_specific_trigger_get_id( $specific_id, $trigger = '', $args = array() ) {

    switch ( $trigger ) {
        case 'gamipress_ld_complete_specific_quiz':
        case 'gamipress_ld_complete_specific_quiz_grade':
        case 'gamipress_ld_complete_specific_quiz_max_grade':
        case 'gamipress_ld_pass_specific_quiz':
        case 'gamipress_ld_fail_specific_quiz':
        case 'gamipress_ld_complete_specific_topic':
        case 'gamipress_ld_complete_specific_lesson':
        case 'gamipress_ld_complete_specific_course':
            $specific_id = $args[0];
            break;
    }

    return $specific_id;
}
add_filter( 'gamipress_specific_trigger_get_id', 'gamipress_ld_specific_trigger_get_id', 10, 3 );

/**
 * Extended meta data for event trigger logging
 *
 * @since 1.0.0
 *
 * @param array 	$log_meta
 * @param integer 	$user_id
 * @param string 	$trigger
 * @param integer 	$site_id
 * @param array 	$args
 *
 * @return array
 */
function gamipress_ld_log_event_trigger_meta_data( $log_meta, $user_id, $trigger, $site_id, $args ) {

    switch ( $trigger ) {
        case 'gamipress_ld_complete_quiz':
        case 'gamipress_ld_complete_specific_quiz':
        case 'gamipress_ld_pass_quiz':
        case 'gamipress_ld_pass_specific_quiz':
        case 'gamipress_ld_fail_quiz':
        case 'gamipress_ld_fail_specific_quiz':
            // Add the quiz and course IDs
            $log_meta['quiz_id'] = $args[0];
            $log_meta['course_id'] = $args[2];
            break;
        case 'gamipress_ld_complete_quiz_grade':
        case 'gamipress_ld_complete_specific_quiz_grade':
        case 'gamipress_ld_complete_quiz_max_grade':
        case 'gamipress_ld_complete_specific_quiz_max_grade':
            // Add the quiz and course IDs
            $log_meta['quiz_id'] = $args[0];
            $log_meta['course_id'] = $args[2];
            $log_meta['score'] = $args[3];
            break;
        case 'gamipress_ld_complete_topic':
        case 'gamipress_ld_complete_specific_topic':
            // Add the topic, lesson and course IDs
            $log_meta['topic_id'] = $args[0];
            $log_meta['lesson_id'] = $args[2];
            $log_meta['course_id'] = $args[3];
            break;
        case 'gamipress_ld_complete_lesson':
        case 'gamipress_ld_complete_specific_lesson':
            // Add the lesson and course IDs
            $log_meta['lesson_id'] = $args[0];
            $log_meta['course_id'] = $args[2];
            break;
        case 'gamipress_ld_complete_course':
        case 'gamipress_ld_complete_specific_course':
            // Add the course ID
            $log_meta['course_id'] = $args[0];
            break;
    }

    return $log_meta;
}
add_filter( 'gamipress_log_event_trigger_meta_data', 'gamipress_ld_log_event_trigger_meta_data', 10, 5 );